@font-face {
    font-family: Source Code Pro;
    font-style: regular;
    font-weight: 400;
    src: url(./SourceCodePro-Regular.ttf) format("truetype")
}

@font-face {
    font-family: Source Code Pro;
    font-style: regular;
    font-weight: 600;
    src: url(./SourceCodePro-SemiBold.ttf) format("truetype")
}

@font-face {
    font-family: Source Code Pro;
    font-style: regular;
    font-weight: 700;
    src: url(./SourceCodePro-Bold.ttf) format("truetype")
}
